<?php

namespace Drupal\Tests\ip_info\Unit\Services;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\ip_info\Services\IpInfo;
use Drupal\Tests\UnitTestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * @coversDefaultClass \Drupal\ip_info\Services\IpInfo
 * @group ip_info
 */
class IpInfoTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface|\Prophecy\Prophecy\ObjectProphecy
   */
  protected $loggerFactory;

  /**
   * The IpInfo service.
   *
   * @var \Drupal\ip_info\Services\IpInfo
   */
  protected $ipInfo;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $this->loggerFactory = $this->prophesize(LoggerChannelFactoryInterface::class);

    $mock = new MockHandler();
    $handler = HandlerStack::create($mock);
    $client = new Client(['handler' => $handler]);

    $this->ipInfo = new IpInfo($client, $this->configFactory->reveal(), $this->loggerFactory->reveal());
  }

  /**
   * @covers ::__construct
   */
  public function testConstructor(): void {
    $this->assertInstanceOf(IpInfo::class, $this->ipInfo);
  }

  /**
   * @covers ::getIpInformation
   */
  public function testGetIpInformationSuccess(): void {
    $mock = new MockHandler([
      new Response(200, [], '{"country": "US", "city": "New York"}'),
    ]);
    $handler = HandlerStack::create($mock);
    $client = new Client(['handler' => $handler]);
    $ipInfo = new IpInfo($client, $this->configFactory->reveal(), $this->loggerFactory->reveal());

    $result = $ipInfo->getIpInformation('8.8.8.8', 'https://example.com/ip', 'test_api');
    $this->assertEquals(['country' => 'US', 'city' => 'New York'], $result);
  }

  /**
   * @covers ::getApiKey
   */
  public function testGetApiKeyFromConfig(): void {
    $config = $this->prophesize(Config::class);
    $config->get('ipinfo_io_token')->willReturn('token_from_config');
    $this->configFactory->get('ip_info.settings')->willReturn($config->reveal());

    $ipInfo = new IpInfo(new Client(), $this->configFactory->reveal(), $this->loggerFactory->reveal());

    $key = $ipInfo->getApiKey('ipinfo_io_token');
    $this->assertEquals('token_from_config', $key);
  }

  /**
   * @covers ::isEnabled
   */
  public function testIsEnabledFromConfig(): void {
    $config = $this->prophesize(Config::class);
    $config->get('ipapi_co_enable')->willReturn(FALSE);
    $this->configFactory->get('ip_info.settings')->willReturn($config->reveal());

    $ipInfo = new IpInfo(new Client(), $this->configFactory->reveal(), $this->loggerFactory->reveal());
    $this->assertFalse($ipInfo->isEnabled('ipapi_co_enable'));
  }

}
